#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import joker.filesys.cas
from joker.filesys import utils
from joker.filesys.utils import Pathlike, checksum_hexdigest


class ContentAddressedStorage(joker.filesys.cas.ContentAddressedStorage):
    def seize(self, path: Pathlike) -> str:
        cid = checksum_hexdigest(path, "sha256")
        utils.moves(path, self.locate(cid))
        return cid



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import datetime
import logging
import mimetypes
import os
from typing import Iterable, TypedDict

import pymongo.errors
from flask import request

from cascadis.environ import GlobalInterface
from cascadis.solutions.inference import get_or_infer_content_type
from cascadis.solutions.legacy import (
    infer_mimetype_from_upload,
    infer_mimetype_with_file_command,
)

gi = GlobalInterface()
_logger = logging.getLogger(__name__)


def query_by_file_id(file_id):
    coll = gi.mongodb.get_collection("file_registry")
    cursor = coll.find({"file_id": file_id})
    return list(cursor)


def legacy_register(file_id, path, ttl=86400):
    coll = gi.mongodb["file_registry"]
    now = datetime.datetime.now()
    expired_at = now + datetime.timedelta(seconds=ttl)
    record = {
        "file_id": file_id,
        "path": path,
        "created_at": now,
        "expired_at": expired_at,
    }
    try:
        ir = coll.insert_one(record)
    except pymongo.errors.DuplicateKeyError:
        return
    return ir.inserted_id


def register_and_query(file_id, path):
    reg_id = legacy_register(file_id, path)
    coll = gi.mongodb.get_collection("file_registry")
    for rec in coll.find({"_id": reg_id}):
        rec["_new"] = True
        yield rec
    yield from coll.find({"file_id": file_id, "_id": {"$ne": reg_id}})


def read_uploading_file(chunksize=16384) -> Iterable[bytes]:
    fup = request.files.get("file")
    while chunk := fup.stream.read(chunksize):
        yield chunk


class UploadRecord(TypedDict):
    cid: str
    method: str
    username: str
    mimetype: str | None
    filename: str | None


def register_object(cid: str, mimetype: str):
    coll = gi.mongodb["objects"]
    coll.update_one(
        {"_id": cid},
        {"$set": {"mimetype": mimetype, "_id": cid}},
        upsert=True,
    )


def register_http_upload(cid: str):
    coll = gi.mongodb["uploads"]
    fup = request.files.get("file")
    record: UploadRecord = {
        "cid": cid,
        "method": "http",
        "username": "admin",
        "mimetype": fup.mimetype,
        "filename": fup.filename,
    }
    coll.insert_one(record)
    register_object(cid, fup.mimetype)


def register_cli_upload(cid: str, filename: str):
    coll = gi.mongodb["uploads"]
    mimetype = mimetypes.guess_type(filename)[0]
    record: UploadRecord = {
        "cid": cid,
        "method": "cli",
        "username": os.getlogin(),
        "mimetype": mimetype,
        "filename": filename,
    }
    coll.insert_one(record)
    if mimetype:
        register_object(cid, mimetype)


if __name__ == "__main__":
    print(
        infer_mimetype_with_file_command(
            "3543069d447f8ef042b5f0500e36a9b787f9288a064dd38905f4147169f9c476"
        )
    )
    print(
        infer_mimetype_from_upload(
            "3543069d447f8ef042b5f0500e36a9b787f9288a064dd38905f4147169f9c476"
        )
    )



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import re
import subprocess

from cascadis.environ import GlobalInterface

gi = GlobalInterface()

# e.g. "pixels.jpg: image/jpeg; charset=binary"
_file_cmd_output_regex = re.compile(r":\s*(\w+/\w+; charset=\w+)$")


def infer_mimetype_from_upload(cid: str) -> str:
    coll = gi.mongodb["uploads"]
    record = coll.find_one(
        {"cid": cid},
        sort=[("_id", -1)],
        projection=["mimetype"],
    )
    if record:
        return record["mimetype"]


def infer_mimetype_with_file_command(cid: str) -> str:
    path = gi.cas.locate(cid)
    cmd = ["file", "--mime-type", str(path)]
    proc = subprocess.run(cmd, capture_output=True, text=True)
    return proc.stdout.split(":", maxsplit=1)[-1]


def infer_content_type_with_file_command(cid: str) -> str:
    path = gi.cas.locate(cid)
    cmd = ["file", "--mime", str(path)]
    proc = subprocess.run(cmd, capture_output=True, text=True)
    mat = _file_cmd_output_regex.search(proc.stdout.strip())
    if mat is None:
        return "application/octet-stream"
    return mat.group(1)


def infer_mimetype(cid: str):
    if mimetype := infer_mimetype_from_upload(cid):
        return mimetype
    if mimetype := infer_mimetype_with_file_command(cid):
        return mimetype
    return "application/octet-stream"





#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import argparse
import os
import sys
from concurrent.futures import ThreadPoolExecutor

from joker.cast.iterative import chunkwize

from cascadis.environ import GlobalInterface
from cascadis.solutions import register_cli_upload
from cascadis.utils import find_regular_files

gi = GlobalInterface()


class _CommandContext:
    def __init__(self, delete=False):
        self.delete = delete

    def _put_file_into_cas(self, path: str):
        if self.delete and os.stat(path).st_dev == os.stat(gi.files).st_dev:
            return gi.cas.seize(path)
        with open(path, "rb") as fin:
            content = fin.read()
            cid = gi.cas.save([content])
        if self.delete:
            os.remove(path)
        return cid

    def put_file_into_cas(self, path: str):
        path = os.path.abspath(path)
        cid = self._put_file_into_cas(path)
        print(cid, path)
        register_cli_upload(cid, path)
        return cid

    def put_files_in_dir_into_cas(self, dirpath):
        executor = ThreadPoolExecutor(max_workers=10)
        paths = find_regular_files(dirpath)
        for batch in chunkwize(1000, paths):
            executor.map(self.put_file_into_cas, batch)


def _main(paths: list[str], delete=False):
    cc = _CommandContext(delete=delete)
    for path in paths:
        if os.path.isdir(path):
            cc.put_files_in_dir_into_cas(path)
        else:
            cc.put_file_into_cas(path)


def main(prog: str, args: list[str]):
    desc = "Put files into Cascadis."
    pr = argparse.ArgumentParser(prog=prog, description=desc)
    pr.add_argument(
        "-D",
        "--delete",
        action="store_true",
        help="delete source files",
    )
    pr.add_argument(
        "files",
        metavar="path",
        nargs="*",
        help="source file",
    )
    ns = pr.parse_args(args)
    _main(ns.files, delete=ns.delete)


if __name__ == "__main__":
    main(sys.argv[0], sys.argv[1:])



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

from functools import wraps

import flask
from joker.flasky.auth import HashedPassword
from pymongo.errors import DuplicateKeyError
from volkanic.errors import BusinessError

from cascadis.environ import GlobalInterface

gi = GlobalInterface()


class RequestBoundSingletonMeta(type):
    def __call__(cls, *args, **kwargs):
        import flask

        cache = flask.g.setdefault("request_bound_cache", {})
        try:
            return cache[cls]
        except KeyError:
            obj = super().__call__(*args, **kwargs)
            return cache.setdefault(cls, obj)


_builtin_users = {
    "admin": {
        "username": "admin",
        "password": (
            "50cbda2340ad708f307d1d9a7c084959bee68e6749651305ef44bf09c23e2888"
            ":sha256:cascadis"
        ),
    }
}


class LoginInterface(metaclass=RequestBoundSingletonMeta):
    def __init__(self, username: str, userinfo=None):
        self.username = username
        if userinfo:
            self.userinfo = userinfo  # noqa

    @classmethod
    def check(cls):
        if username := flask.session.get("username"):
            return cls(username)
        elif username := gi.conf.get("_auto_login_username"):
            return cls(username)
        else:
            raise BusinessError("You are not logged-in", "BE-0315")

    @staticmethod
    def install_builtin_users():
        coll = gi.mongodb.get_collection("users")
        if coll.find_one(projection=[]):
            return
        coll.create_index("username", unique=True)
        for userinfo in _builtin_users.values():
            try:
                coll.insert_one(userinfo)
            except DuplicateKeyError:
                print("failed:", userinfo)

    @classmethod
    def query(cls, username: str) -> dict | None:
        coll = gi.mongodb.get_collection("users")
        return coll.find_one({"username": username})

    @classmethod
    def login(cls, username: str, password: str):
        userinfo = cls.query(username)
        err_login = BusinessError("Wrong username or password", "LoginFailed")
        if not userinfo:
            raise err_login
        h_password = userinfo.pop("password")
        hp = HashedPassword.parse(h_password)
        if not hp.verify(password):
            raise err_login
        flask.session["username"] = username
        return cls(username, userinfo)


def login_required(func):
    @wraps(func)
    def decorated_func(*args, **kwargs):
        LoginInterface.check()
        return func(*args, **kwargs)

    return decorated_func


def set_protection_level(level: int):
    """
    If a view function has protection level higher
    than gi.conf['accessibility'], the user has to
    log in to access it.
    """

    def _decorator(func):
        @wraps(func)
        def decorated_func(*args, **kwargs):
            if gi.conf["accessibility"] < level:
                LoginInterface.check()
            return func(*args, **kwargs)

        return decorated_func

    return _decorator



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import argparse
import os
import sys
from concurrent.futures import ThreadPoolExecutor

from joker.cast.iterative import chunkwize
from volkanic.utils import printerr

from cascadis.environ import GlobalInterface

gi = GlobalInterface()


class _CommandContext:
    def __init__(self, overwrite=False):
        self.overwrite = overwrite

    def get_file_from_cas(self, cid: str, path: str):
        dir_ = os.path.split(path)[0]
        os.makedirs(dir_, exist_ok=True)
        exists = os.path.exists(path)
        if not self.overwrite and exists:
            print(cid, "SKIPPED", path)
            return
        with open(path, "wb") as fout:
            for chunk in gi.cas.load(cid):
                fout.write(chunk)
        action = "OVERWRITTEN" if exists else "CREATED"
        print(cid, action, path)

    def _get_file_from_cas_with_one_arg(self, pair: tuple[str, str]):
        self.get_file_from_cas(*pair)

    def get_files_from_cas(self, pairs: list[tuple[str, str]]):
        executor = ThreadPoolExecutor(max_workers=10)
        for batch in chunkwize(1000, pairs):
            executor.map(self._get_file_from_cas_with_one_arg, batch)


def main(_prog: str, args: list[str]):
    desc = "Get files from Cascadis."
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument(
        "-O",
        "--overwrite",
        action="store_true",
        help="overwrite existing files",
    )
    parser.add_argument("cid", help="content identifier")
    parser.add_argument("path", help="file to be created")
    parser.add_argument(
        "more",
        nargs="*",
        default=[],
        help="more cid and path pairs",
    )
    ns = parser.parse_args(args)
    if len(ns.more) % 2:
        printerr("error: wrong number of arguments")
        sys.exit(1)
    pairs = chunkwize(2, [ns.cid, ns.path] + ns.more)
    _CommandContext(ns.overwrite).get_files_from_cas(list(pairs))


if __name__ == "__main__":
    main(sys.argv[0], sys.argv[1:])



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import os
import time

from volkanic.utils import ignore_arguments

from cascadis.auth import LoginInterface
from cascadis.environ import GlobalInterface

gi = GlobalInterface()


def remove_legacy_files():
    ttl = gi.conf["cache_ttl"]
    if not ttl:
        return
    now = time.time()
    for path in gi.cas.iter_paths():
        mtime = os.path.getmtime(path)
        if now - mtime > ttl:
            os.remove(path)


@ignore_arguments
def main():
    LoginInterface.install_builtin_users()
    while True:
        time.sleep(3600)
        remove_legacy_files()



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import threading

from magic import Magic

from cascadis.environ import GlobalInterface

gi = GlobalInterface()
_thread_local = threading.local()
_thread_local.magic = Magic(mime=True, mime_encoding=True)


def _get_magic() -> Magic:
    try:
        return _thread_local.magic
    except AttributeError:
        _thread_local.magic = Magic(mime=True, mime_encoding=True)
        return _thread_local.magic


def infer_content_type(cid: str) -> str:
    path = gi.cas.locate(cid)
    return _get_magic().from_file(path)


def get_or_infer_content_type(cid: str) -> str:
    record = gi.mongodb["objects"].find_one(
        {"_id": cid, "content_type": {"$exists": True}},
    )
    if record:
        return record["content_type"]
    content_type = infer_content_type(cid)
    gi.mongodb["objects"].update_one(
        {"_id": cid},
        {"$set": {"content_type": content_type}},
        upsert=True,
    )
    return content_type



#!/usr/bin/env python3
# coding: utf-8

import os
import sys

from bson import ObjectId

from cascadis.environ import GlobalInterface
from cascadis.solutions import register_and_query

gi = GlobalInterface()


def oid_to_date(oid: ObjectId):
    dt = oid.generation_time
    return dt.strftime("%Y-%m-%d_%H:%M:%S")


def fmt_register_record(rec: dict):
    _id = rec["_id"]
    date = oid_to_date(_id)
    file_id = rec["file_id"]
    path = rec["path"]
    path_flag = "+" if os.path.isfile(path) else "-"
    new_flag = "*" if rec.get("_new") else " "
    return f"{file_id} {new_flag}{date} {path_flag} {path}"


def read_file(path):
    blksize = 1024 * 1024
    with open(path, "rb") as fin:
        blk = fin.read(blksize)
        while blk:
            yield blk
            blk = fin.read(blksize)


def main():
    for path in sys.argv[1:]:
        path = os.path.abspath(path)
        file_id = gi.cas.save(read_file(path))
        for rec in register_and_query(file_id, path):
            print(fmt_register_record(rec))
        print()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import sys

from cascadis.environ import GlobalInterface

gi = GlobalInterface()


def main(_prog, _args):
    from IPython import start_ipython

    gi.setup_logging('INFO')
    start_ipython([], user_ns=globals())


if __name__ == "__main__":
    main(sys.argv[0], sys.argv[1:])



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import dataclasses
import urllib.parse

import flask
import requests
from flask import request, Response


@dataclasses.dataclass
class Proxy:
    base_url: str
    default_ttl: int = 5

    def get_ttl(self) -> int:
        key = "X-Proxy-TTL"
        return request.headers.get(key, default=self.default_ttl, type=int)

    def fetch(self, filename: str) -> requests.Response | None:
        url = urllib.parse.urljoin(self.base_url, f"files/{filename}")
        ttl = self.get_ttl() - 1
        if not ttl:
            return
        resp = requests.get(url, headers={"X-Proxy-TTL": str(ttl)})
        if resp.status_code == 404:
            return
        return resp

    def forward(self, filename: str) -> Response:
        resp = self.fetch(filename)
        if resp is None:
            return flask.abort(404)
        ret_resp = Response(
            resp.content,
            content_type=resp.headers["Content-Type"],
        )
        if disposition := resp.headers.get("Content-Disposition"):
            ret_resp.headers["Content-Disposition"] = disposition
        return ret_resp


def fmt_content_disposition(filename: str, attachment=False) -> str:
    quoted = urllib.parse.quote(filename)
    kind = "attachment" if attachment else "inline"
    return f'''{kind}; filename="{quoted}"; filename*="UTF-8''{quoted}"'''


def fmt_content_disposition_attachment(filename: str) -> str:
    return fmt_content_disposition(filename, attachment=True)



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import logging
import os
from functools import cached_property
from pathlib import Path

import volkanic
from joker.redis import ErrorInterface
from pymongo import MongoClient
from pymongo.database import Database
from redis import Redis
from volkanic.utils import ignore_arguments, indented_json_print

from cascadis.cas import ContentAddressedStorage

_logger = logging.getLogger(__name__)


class GlobalInterface(volkanic.GlobalInterface):
    package_name = "cascadis"
    default_config = {
        "cache_ttl": 0,
        "proxy_ttl": 5,
        "mongo": {},
        "redis": {},
        "accessibility": 1,
        "auth_tokens": [],
        "data_dir": "/data",
        "dist_dirs": {},
        "upstream": None,
        "use_nginx": False,
        "test_account": {},
    }

    @cached_property
    def cas(self) -> ContentAddressedStorage:
        kwargs = {
            "base_dir": self.files,
            "dist_dirs": self.conf["dist_dirs"],
        }
        return ContentAddressedStorage(**kwargs)

    @cached_property
    def files(self) -> Path:
        path = Path(self.conf["data_dir"]) / f"{self.project_name}.files"
        path.mkdir(parents=True, exist_ok=True)
        return path

    @cached_property
    def _mongodb_and_pid(self) -> tuple[Database, int]:
        cfg = self.conf["mongo"]
        cfg = cfg.copy()
        cfg.setdefault("directConnection", True)
        _logger.info("using mongo cfg %s", cfg)
        client = MongoClient(**cfg)
        return client.get_database(self.identifier), os.getpid()

    @property
    def mongodb(self):
        database, pid = self._mongodb_and_pid
        # for a new process, create a new MongoClient instance
        if pid != os.getpid():
            self.__dict__.pop("_mongodb_and_pid")
            database = self._mongodb_and_pid[0]
        return database

    @cached_property
    def error_interface(self) -> ErrorInterface:
        return ErrorInterface(self.redis, self.project_name)

    @cached_property
    def redis(self) -> Redis:
        return Redis(**self.conf["redis"])


@ignore_arguments
def main():
    gi = GlobalInterface()
    indented_json_print(gi.conf)
    print("files:", gi.files)



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import logging
import mimetypes
import os.path

import flask
from flask import Blueprint, request, Response

from cascadis.auth import set_protection_level, LoginInterface
from cascadis.environ import GlobalInterface
from cascadis.solutions import register_http_upload
from cascadis.solutions.sessions import Session
from cascadis.viewutils import (
    Proxy, fmt_content_disposition
)

gi = GlobalInterface()
bp = Blueprint("cas", __name__)
_logger = logging.getLogger(__name__)


def _read_uploading_file_in_chunks(chunksize=16384):
    fup = request.files.get("file")
    while chunk := fup.stream.read(chunksize):
        yield chunk


@bp.route("/")
@set_protection_level(1)
def api_index():
    import cascadis

    return {
        "code": 0,
        "data": {
            "project": gi.project_name,
            "version": cascadis.__version__,
            "homepage": "https://github.com/frozflame/cascadis",
            "cache_ttl": gi.conf["cache_ttl"],
        },
    }


@bp.route("/cas/login", methods=["GET", "POST"])
@bp.route("/cascadis/login", methods=["GET", "POST"])
def api_login():
    if request.method == "GET":
        # TODO: or use joker.flasky.pages?
        # pages.respond_login_page(**gi.conf["test_account"])
        return flask.render_template("login.html", **gi.conf["test_account"])
    LoginInterface.install_builtin_users()
    username = request.form["username"]
    password = request.form["password"]
    li = LoginInterface.login(username, password)
    return {"code": 0, "data": li.userinfo}


@bp.route("/cas/files", methods=["GET", "POST"])
@bp.route("/cascadis/files", methods=["GET", "POST"])
@set_protection_level(2)
def api_upload():
    # if request.method == "GET":
    #     return pages.respond_upload_page()
    # chunks = read_uploading_file()
    if request.method == "GET":
        return flask.render_template("upload.html")
    chunks = _read_uploading_file_in_chunks()
    cid = gi.cas.save(chunks)
    register_http_upload(cid)
    return {"code": 0, "data": cid}


def _respond_xaccel_redirect(key: str, ext: str, contenttype: str):
    # nginx: http://wiki.nginx.org/NginxXSendfile
    resp = flask.make_response()
    rel_path = gi.cas.locate(key).relative_to(gi.files)
    headers = {
        "X-Accel-Redirect": str(rel_path),
        "Cache-Control": "no-cache",
        "Content-Type": contenttype,
        "Content-Disposition": f"attachment; filename={key}{ext}",
    }
    resp.headers.update(headers)
    return resp


@bp.route("/cas/files/<cname>")
@bp.route("/cascadis/files/<cname>")
@bp.route("/cas/c/<cname>")
@bp.route("/cascadis/content/<cname>")
@set_protection_level(1)
def api_download(cname: str):
    # TODO: support Content-Length on GET/HEAD
    # TODO: use HEAD method on HEAD proxy
    cid, ext = os.path.splitext(cname)
    if len(cid) != 64:
        return flask.abort(404)
    if not gi.cas.exists(cid):
        if upstream := gi.conf["upstream"]:
            proxy = Proxy(upstream, gi.conf["proxy_ttl"])
            return proxy.forward(cname)
        return flask.abort(404)
    default_mimetype = "application/octet-stream"
    filename = request.args.get("filename", cname)
    mimetype = mimetypes.guess_type(filename)[0] or default_mimetype
    if gi.conf["use_nginx"]:
        return _respond_xaccel_redirect(cid, ext, mimetype)
    if request.method == "HEAD":
        chunks = []
    else:
        chunks = gi.cas.load(cid)
    # https://flask.palletsprojects.com/en/2.1.x/patterns/streaming/#basic-usage
    resp = Response(chunks, mimetype=mimetype)
    if mimetype == default_mimetype:
        resp.headers["Content-Disposition"] = "inline"
    elif filename != cname:
        content_dispos = fmt_content_disposition(filename, attachment=True)
        resp.headers["Content-Disposition"] = content_dispos
    return resp


@bp.route("/cas/s/<session_id>")
@bp.route("/cascadis/session/<session_id>")
def view_public(session_id):
    try:
        session = Session.load(session_id)
    except FileNotFoundError:
        return flask.abort(404)
    chunks = gi.cas.load(session.cid)
    print(session, "----")
    return Response(chunks, content_type=session.content_type)



#!/usr/bin/env python3
# coding: utf-8
import volkanic

cmddef = """
api     cascadis.api:main
atx     cascadis.atx:main
conf    cascadis.environ:main
get     cascadis.cli.get:main
misc    cascadis.misc:main
put     cascadis.cli.put:main
shell   cascadis.cli.shell:main
"""

_prog = 'python3 -m cascadis'
registry = volkanic.CommandRegistry.from_cmddef(cmddef, _prog)

if __name__ == '__main__':
    registry()



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import os


def find_regular_files(dirpath, **kwargs):
    for root, dirs, files in os.walk(dirpath, **kwargs):
        for name in files:
            path = os.path.join(root, name)
            if os.path.isfile(path):
                yield path



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import argparse
import sys
import time
import traceback

import flask

# noinspection PyPackageRequirements
import werkzeug.exceptions
from flask import request
from flask.json.provider import DefaultJSONProvider
from joker.flasky.viewutils import (
    json_default_strict,
    chain_json_default_functions,
)
from volkanic.errors import KnownError
from volkanic.introspect import ErrorInfo

import cascadis.views
from cascadis.environ import GlobalInterface

gi = GlobalInterface()


class JSONProvider(DefaultJSONProvider):
    default = chain_json_default_functions(
        json_default_strict,
        DefaultJSONProvider.default,
        str,
    )


class FlaskApplication(flask.Flask):
    json_provider_class = JSONProvider


app = FlaskApplication(__name__)
app.secret_key = "TfT2T6UA1BTdk7HCIGmpvcmFTla45jYE2QKIiys4AAEGQxAt09Trc71JyM"


@app.errorhandler(KnownError)
def on_known_error(error: KnownError):
    if request.files:
        time.sleep(1)
    return error.to_dict()


@app.errorhandler(Exception)
def on_other_error(_error: Exception):
    # https://flask.palletsprojects.com/en/2.0.x/errorhandling/#generic-exception-handlers
    if isinstance(_error, werkzeug.exceptions.HTTPException):
        return _error
    traceback.print_exc()
    errinfo = ErrorInfo()
    return errinfo.to_dict()


# app.use_default_error_handlers(gi.error_interface)
app.register_blueprint(cascadis.views.bp)


def main(prog: str, args: list[str]):
    msg = (
        "For production, run with gunicorn or uwsgi:\n"
        "gunicorn -w 8 cascadis.api:app -b 0.0.0.0:16000"
    )
    print(msg)
    desc = "run cascadis web service"
    pr = argparse.ArgumentParser(prog=prog, description=desc, add_help=False)
    add = pr.add_argument
    add("--help", action="help", help="show this help message and exit")
    add("-h", "--host", help="hostname", default="127.0.0.1")
    add("-p", "--port", type=int, help="port number", default=16000)
    ns = pr.parse_args(args)
    app.run(ns.host, ns.port, debug=True)


if __name__ == "__main__":
    main(sys.argv[0], sys.argv[1:])



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import dataclasses
import datetime
import random
from string import ascii_uppercase
from typing import Self

from cascadis.solutions import gi, _logger, get_or_infer_content_type


def create_indexes():
    c = gi.mongodb["sessions"]
    c.create_index("session_id", unique=True)
    c.create_index("expires_at", expireAfterSeconds=300)


@dataclasses.dataclass
class Session:
    cid: str
    # TODO: rename to sid?
    sid: str
    content_type: str
    filename: str
    expires_at: datetime.datetime

    @classmethod
    def load(cls, sid: str) -> Self:
        filtr = {
            "sid": sid,
            # do not rely on TTL index of mongodb
            "expires_at": {
                "$gt": datetime.datetime.now(),
            },
        }
        c = gi.mongodb["sessions"]
        record = c.find_one(filtr, projection={"_id": False})
        if record is None:
            raise FileNotFoundError(f"session not found: {sid}")
        return cls(**record)

    def save(self):
        record = dataclasses.asdict(self)
        ir = gi.mongodb["sessions"].insert_one(record)
        _logger.info("session saved: %s, %s", ir.inserted_id, self.sid)

    @classmethod
    def create(
        cls,
        cid: str,
        content_type: str = None,
        filename: str = None,
        ttl: int = 60,
    ) -> Self:
        if content_type is None:
            content_type = get_or_infer_content_type(cid)
        session_id = "".join(random.choices(ascii_uppercase, k=20))
        expires_at = datetime.datetime.now() + datetime.timedelta(seconds=ttl)
        return cls(cid, session_id, content_type, filename, expires_at)

    @staticmethod
    def remove_expired():
        """an alternative way to remove expired record"""
        gi.mongodb["sessions"].delete_many(
            {"expires_at": {"$lt": datetime.datetime.now()}}
        )



#!/usr/bin/env python3
# coding: utf-8
import os
import re

import setuptools
from setuptools import find_packages

# CAUTION:
# Do NOT import your package from your setup.py

_nsp = ''
_pkg = 'cascadis'
_desc = ''
_names = [_nsp, _pkg]
_names = [s for s in _names if s]


def read(filename):
    with open(filename) as f:
        return f.read()


def _find_version():
    names = _names + ['__init__.py']
    path = os.path.join(*names)
    root = os.path.dirname(__file__)
    path = os.path.join(root, path)
    regex = re.compile(
        r'''^__version__\s*=\s*('|"|'{3}|"{3})([.\w]+)\1\s*(#|$)'''
    )
    with open(path) as fin:
        for line in fin:
            line = line.strip()
            if not line or line.startswith('#'):
                continue
            mat = regex.match(line)
            if mat:
                return mat.groups()[1]
    raise ValueError('__version__ definition not found')


config = {
    'name': 'cascadis',
    'version': _find_version(),
    'description': 'A simple content-addressed storage service.',
    'keywords': '',
    'url': 'https://github.com/frozflame/cascadis',
    'author': 'frozflame',
    'author_email': 'frozflame@outlook.com',
    'license': "GNU General Public License (GPL)",
    'packages': find_packages(include=['cascadis', 'cascadis.*']),
    'zip_safe': False,
    'install_requires': read("requirements.txt"),
    'entry_points': {
        'console_scripts': [
            'cas = cascadis.__main__:registry',
            'cascadis = cascadis.__main__:registry',
        ]
    },
    'classifiers': [
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
    ],
    # ensure copy static file to runtime directory
    'include_package_data': True,
    'long_description': read('README.md'),
    'long_description_content_type': "text/markdown",
}

if _nsp:
    config['namespace_packages'] = [_nsp]

setuptools.setup(**config)
